//
// Skinned Mesh Effect file 
// Copyright (c) 2000-2002 Microsoft Corporation. All rights reserved.
//
texture tDiffuse;
texture tNormal;
 
float3 vSpecular  = {0.0f, 0.0f, -1.0f};    //light Direction 
float3 vDiffuse = {-0.60f, 0.60f, -0.6f}; // Light Diffuse
float4 cAmbient : MATERIALAMBIENT = {0.3f, 0.3f, 0.3f, 1.0f};
float4 cDiffuse : MATERIALDIFFUSE = {0.8f, 0.8f, 0.7f, 1.0f};
float4 cSpecular : MATERIALDIFFUSE = {0.9f, 0.9f, 0.8f, 1.0f};

// Matrix Pallette
static const int MAX_MATRICES = 26;
float4x3    mWorldMatrixArray[MAX_MATRICES] : WORLDMATRIXARRAY;
float4x4    mViewProj : VIEWPROJECTION;

///////////////////////////////////////////////////////
struct VS_INPUT
{
    float4  vPosition        : POSITION;
    float4  vBlendWeights    : BLENDWEIGHT;
    float4  vBlendIndices    : BLENDINDICES;
    float3  vNormal          : NORMAL;
    float3  vTangent         : TANGENT;
    float3  vBinormal        : BINORMAL;
    float2  vTexCoords       : TEXCOORD0;
};

struct VS_OUTPUT
{
    float4  vPosition     : POSITION;
    float3  vDiffuse      : COLOR0;
    float3  vSpecular     : COLOR1;
    float2  vTexCoords0   : TEXCOORD0;
    float2  vTexCoords1   : TEXCOORD1;
};

struct PS_OUTPUT
{
    float4 vColor           : COLOR0;
};

float3 Diffuse(float3 vNormal)
{
    float CosTheta;
    
    // N.L Clamped
    CosTheta = max(0.0f, dot(vNormal, vSpecular .xyz));
       
    // propogate scalar result to vector
    return (CosTheta);
}


VS_OUTPUT VShade(VS_INPUT i, uniform int NumBones)
{
    VS_OUTPUT   o;
    float3      vPosition = 0.0f;
    float3      vNormal = 0.0f;    
    float3      vTangent = 0.0f;    
    float3      vBinormal = 0.0f;    
    float       LastWeight = 0.0f;
     
    // Compensate for lack of UBYTE4 on Geforce3
    int4 IndexVector = D3DCOLORtoUBYTE4(i.vBlendIndices);

    // cast the vectors to arrays for use in the for loop below
    float BlendWeightsArray[4] = (float[4])i.vBlendWeights;
    int   IndexArray[4]        = (int[4])IndexVector;
    
    // calculate the pos/normal using the "normal" weights 
    //        and accumulate the weights to calculate the last weight
    for (int iBone = 0; iBone < NumBones-1; iBone++)
    {
        LastWeight = LastWeight + BlendWeightsArray[iBone];
        
        vPosition += mul(i.vPosition, mWorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
        vNormal += mul(i.vNormal, mWorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
        vTangent += mul(i.vTangent, mWorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
        //vBinormal += mul(i.vBinormal, mWorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
    }
    LastWeight = 1.0f - LastWeight; 

    // Now that we have the calculated weight, add in the final influence
    vPosition += (mul(i.vPosition, mWorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight);
    vNormal += (mul(i.vNormal, mWorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight); 
    vTangent += (mul(i.vTangent, mWorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight); 
    //vBinormal += (mul(i.vBinormal, mWorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight); 
    
    // Faster way to obtain Binormal but more imprecise: It forces a orthogonal vector in the mLight matrix 
    vBinormal = -cross(vNormal, vTangent);
    
    // Transformation matrix for texture space normal mapping
    float3x3 mLight =  
        float3x3(vTangent.x, vBinormal.x, vNormal.x,
                 vTangent.y, vBinormal.y, vNormal.y, 
                 vTangent.z, vBinormal.z, vNormal.z);

    // transform position from world space into view and then projection space
    o.vPosition = mul(float4(vPosition.xyz, 1.0f), mViewProj);

    // normalize normals
    vNormal = normalize(vNormal);

    // transform diffuse and specular vectors for dot3 in texture space
    o.vDiffuse  = mul(vDiffuse, mLight) * 0.5 + 0.5;
    o.vSpecular = mul(vSpecular, mLight) * 0.35 + 0.35;


    // copy the input texture coordinate through
    o.vTexCoords0 = o.vTexCoords1 = i.vTexCoords.xy;

    return o;
}

int CurNumBones = 2;
VertexShader vsArray[4] = { compile vs_1_1 VShade(1), 
                            compile vs_1_1 VShade(2),
                            compile vs_1_1 VShade(3),
                            compile vs_1_1 VShade(4)
                          };

sampler DiffuseMap = sampler_state
{
Texture   = <tDiffuse>;
MinFilter = LINEAR;
MagFilter = LINEAR;
MipFilter = LINEAR;
};

sampler NormalMap = sampler_state
{
Texture   = <tNormal>;
MinFilter = LINEAR;
MagFilter = LINEAR;
MipFilter = LINEAR;
};

//////////////////////////////////////
// Techniques specs follow
//////////////////////////////////////
technique t0
{
    pass p0
    {
        VertexShader = (vsArray[CurNumBones]);
        
        // Pixel shader
        PixelShader =
            asm
            {
                ps_1_1
                tex t0
                tex t1

               // Specular
                dp3_sat r1, t1_bx2, v1_bx2	// r1 = (2.0 * (NormalMap - 0.5)) * (2.0 * (vSpecular - 0.5))
                dp3_sat r0, t1_bx2, v0_bx2

                mul_sat r1, r1, r0
                mul_sat r1, r1, r1
                mul_sat r1, r1, c2

                // Diffuse + Ambient
                dp3_sat r0, t1, v0_bx2          
                mad_sat r0, r0, c1, c0
                mad_sat r0, r0, t0, r1
            };

        PixelShaderConstant1[0] = (cAmbient);
        PixelShaderConstant1[1] = (cDiffuse);
        PixelShaderConstant1[2] = (cSpecular);

        Sampler[0] = (DiffuseMap);
        Sampler[1] = (NormalMap);
    }
}

